Module: ZipKit::NullWriter

Defined in:
lib/zip_kit/null_writer.rb

Overview

Used when you need to supply a destination IO for some write operations, but want to discard the data (like when estimating the size of a ZIP)

Class Method Summary collapse

Class Method Details

.<<(_) ⇒ self

Parameters:

  • the data to write

Returns:



9
10
11
# File 'lib/zip_kit/null_writer.rb', line 9

def self.<<(_)
  self
end